# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: otap_message.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import wp_global_pb2 as wp__global__pb2
import error_pb2 as error__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='otap_message.proto',
  package='wirepas.proto.gateway_api',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12otap_message.proto\x12\x19wirepas.proto.gateway_api\x1a\x0fwp_global.proto\x1a\x0b\x65rror.proto\"7\n\x0eScratchpadInfo\x12\x0b\n\x03len\x18\x01 \x02(\r\x12\x0b\n\x03\x63rc\x18\x02 \x02(\r\x12\x0b\n\x03seq\x18\x03 \x02(\r\"\xda\x01\n\x19TargetScratchpadAndAction\x12;\n\x06\x61\x63tion\x18\x01 \x02(\x0e\x32+.wirepas.proto.gateway_api.ScratchpadAction\x12\x17\n\x0ftarget_sequence\x18\x02 \x01(\r\x12\x12\n\ntarget_crc\x18\x03 \x01(\r\x12;\n\x05\x64\x65lay\x18\x04 \x01(\x0e\x32*.wirepas.proto.gateway_api.ProcessingDelayH\x00\x12\r\n\x03raw\x18\x05 \x01(\rH\x00\x42\x07\n\x05param\"R\n\x16GetScratchpadStatusReq\x12\x38\n\x06header\x18\x01 \x02(\x0b\x32(.wirepas.proto.gateway_api.RequestHeader\"\xd2\x03\n\x17GetScratchpadStatusResp\x12\x39\n\x06header\x18\x01 \x02(\x0b\x32).wirepas.proto.gateway_api.ResponseHeader\x12\x44\n\x11stored_scratchpad\x18\x02 \x01(\x0b\x32).wirepas.proto.gateway_api.ScratchpadInfo\x12\x42\n\rstored_status\x18\x03 \x01(\x0e\x32+.wirepas.proto.gateway_api.ScratchpadStatus\x12>\n\x0bstored_type\x18\x04 \x01(\x0e\x32).wirepas.proto.gateway_api.ScratchpadType\x12G\n\x14processed_scratchpad\x18\x05 \x01(\x0b\x32).wirepas.proto.gateway_api.ScratchpadInfo\x12\x18\n\x10\x66irmware_area_id\x18\x06 \x01(\r\x12O\n\x11target_and_action\x18\x07 \x01(\x0b\x32\x34.wirepas.proto.gateway_api.TargetScratchpadAndAction\"p\n\x13UploadScratchpadReq\x12\x38\n\x06header\x18\x01 \x02(\x0b\x32(.wirepas.proto.gateway_api.RequestHeader\x12\x0b\n\x03seq\x18\x02 \x02(\r\x12\x12\n\nscratchpad\x18\x03 \x01(\x0c\"Q\n\x14UploadScratchpadResp\x12\x39\n\x06header\x18\x01 \x02(\x0b\x32).wirepas.proto.gateway_api.ResponseHeader\"P\n\x14ProcessScratchpadReq\x12\x38\n\x06header\x18\x01 \x02(\x0b\x32(.wirepas.proto.gateway_api.RequestHeader\"R\n\x15ProcessScratchpadResp\x12\x39\n\x06header\x18\x01 \x02(\x0b\x32).wirepas.proto.gateway_api.ResponseHeader\"\xac\x01\n\x1fSetScratchpadTargetAndActionReq\x12\x38\n\x06header\x18\x01 \x02(\x0b\x32(.wirepas.proto.gateway_api.RequestHeader\x12O\n\x11target_and_action\x18\x02 \x02(\x0b\x32\x34.wirepas.proto.gateway_api.TargetScratchpadAndAction\"]\n SetScratchpadTargetAndActionResp\x12\x39\n\x06header\x18\x01 \x02(\x0b\x32).wirepas.proto.gateway_api.ResponseHeader*5\n\x0eScratchpadType\x12\t\n\x05\x42LANK\x10\x01\x12\x0b\n\x07PRESENT\x10\x02\x12\x0b\n\x07PROCESS\x10\x03*3\n\x10ScratchpadStatus\x12\x0b\n\x07SUCCESS\x10\x01\x12\x07\n\x03NEW\x10\x02\x12\t\n\x05\x45RROR\x10\x03*\x99\x01\n\x10ScratchpadAction\x12\x12\n\x0eUNKNOWN_ACTION\x10\x00\x12\x0b\n\x07NO_OTAP\x10\x01\x12\x12\n\x0ePROPAGATE_ONLY\x10\x02\x12\x19\n\x15PROPAGATE_AND_PROCESS\x10\x03\x12$\n PROPAGATE_AND_PROCESS_WITH_DELAY\x10\x04\x12\x0f\n\x0bLEGACY_OTAP\x10\x05*\x90\x01\n\x0fProcessingDelay\x12\x11\n\rUNKNOWN_DELAY\x10\x00\x12\x0f\n\x0bTEN_MINUTES\x10\x01\x12\x12\n\x0eTHIRTY_MINUTES\x10\x02\x12\x0c\n\x08ONE_HOUR\x10\x03\x12\r\n\tSIX_HOURS\x10\x04\x12\x0b\n\x07ONE_DAY\x10\x05\x12\x0c\n\x08TWO_DAYS\x10\x06\x12\r\n\tFIVE_DAYS\x10\x07'
  ,
  dependencies=[wp__global__pb2.DESCRIPTOR,error__pb2.DESCRIPTOR,])

_SCRATCHPADTYPE = _descriptor.EnumDescriptor(
  name='ScratchpadType',
  full_name='wirepas.proto.gateway_api.ScratchpadType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BLANK', index=0, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRESENT', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROCESS', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1543,
  serialized_end=1596,
)
_sym_db.RegisterEnumDescriptor(_SCRATCHPADTYPE)

ScratchpadType = enum_type_wrapper.EnumTypeWrapper(_SCRATCHPADTYPE)
_SCRATCHPADSTATUS = _descriptor.EnumDescriptor(
  name='ScratchpadStatus',
  full_name='wirepas.proto.gateway_api.ScratchpadStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NEW', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1598,
  serialized_end=1649,
)
_sym_db.RegisterEnumDescriptor(_SCRATCHPADSTATUS)

ScratchpadStatus = enum_type_wrapper.EnumTypeWrapper(_SCRATCHPADSTATUS)
_SCRATCHPADACTION = _descriptor.EnumDescriptor(
  name='ScratchpadAction',
  full_name='wirepas.proto.gateway_api.ScratchpadAction',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ACTION', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_OTAP', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROPAGATE_ONLY', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROPAGATE_AND_PROCESS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROPAGATE_AND_PROCESS_WITH_DELAY', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEGACY_OTAP', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1652,
  serialized_end=1805,
)
_sym_db.RegisterEnumDescriptor(_SCRATCHPADACTION)

ScratchpadAction = enum_type_wrapper.EnumTypeWrapper(_SCRATCHPADACTION)
_PROCESSINGDELAY = _descriptor.EnumDescriptor(
  name='ProcessingDelay',
  full_name='wirepas.proto.gateway_api.ProcessingDelay',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_DELAY', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEN_MINUTES', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='THIRTY_MINUTES', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ONE_HOUR', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIX_HOURS', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ONE_DAY', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TWO_DAYS', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FIVE_DAYS', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1808,
  serialized_end=1952,
)
_sym_db.RegisterEnumDescriptor(_PROCESSINGDELAY)

ProcessingDelay = enum_type_wrapper.EnumTypeWrapper(_PROCESSINGDELAY)
BLANK = 1
PRESENT = 2
PROCESS = 3
SUCCESS = 1
NEW = 2
ERROR = 3
UNKNOWN_ACTION = 0
NO_OTAP = 1
PROPAGATE_ONLY = 2
PROPAGATE_AND_PROCESS = 3
PROPAGATE_AND_PROCESS_WITH_DELAY = 4
LEGACY_OTAP = 5
UNKNOWN_DELAY = 0
TEN_MINUTES = 1
THIRTY_MINUTES = 2
ONE_HOUR = 3
SIX_HOURS = 4
ONE_DAY = 5
TWO_DAYS = 6
FIVE_DAYS = 7



_SCRATCHPADINFO = _descriptor.Descriptor(
  name='ScratchpadInfo',
  full_name='wirepas.proto.gateway_api.ScratchpadInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='len', full_name='wirepas.proto.gateway_api.ScratchpadInfo.len', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='crc', full_name='wirepas.proto.gateway_api.ScratchpadInfo.crc', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq', full_name='wirepas.proto.gateway_api.ScratchpadInfo.seq', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=79,
  serialized_end=134,
)


_TARGETSCRATCHPADANDACTION = _descriptor.Descriptor(
  name='TargetScratchpadAndAction',
  full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction.action', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target_sequence', full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction.target_sequence', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target_crc', full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction.target_crc', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delay', full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction.delay', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw', full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction.raw', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='param', full_name='wirepas.proto.gateway_api.TargetScratchpadAndAction.param',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=137,
  serialized_end=355,
)


_GETSCRATCHPADSTATUSREQ = _descriptor.Descriptor(
  name='GetScratchpadStatusReq',
  full_name='wirepas.proto.gateway_api.GetScratchpadStatusReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.GetScratchpadStatusReq.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=357,
  serialized_end=439,
)


_GETSCRATCHPADSTATUSRESP = _descriptor.Descriptor(
  name='GetScratchpadStatusResp',
  full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stored_scratchpad', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.stored_scratchpad', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stored_status', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.stored_status', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stored_type', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.stored_type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='processed_scratchpad', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.processed_scratchpad', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='firmware_area_id', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.firmware_area_id', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target_and_action', full_name='wirepas.proto.gateway_api.GetScratchpadStatusResp.target_and_action', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=442,
  serialized_end=908,
)


_UPLOADSCRATCHPADREQ = _descriptor.Descriptor(
  name='UploadScratchpadReq',
  full_name='wirepas.proto.gateway_api.UploadScratchpadReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.UploadScratchpadReq.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq', full_name='wirepas.proto.gateway_api.UploadScratchpadReq.seq', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scratchpad', full_name='wirepas.proto.gateway_api.UploadScratchpadReq.scratchpad', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=910,
  serialized_end=1022,
)


_UPLOADSCRATCHPADRESP = _descriptor.Descriptor(
  name='UploadScratchpadResp',
  full_name='wirepas.proto.gateway_api.UploadScratchpadResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.UploadScratchpadResp.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1024,
  serialized_end=1105,
)


_PROCESSSCRATCHPADREQ = _descriptor.Descriptor(
  name='ProcessScratchpadReq',
  full_name='wirepas.proto.gateway_api.ProcessScratchpadReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.ProcessScratchpadReq.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1107,
  serialized_end=1187,
)


_PROCESSSCRATCHPADRESP = _descriptor.Descriptor(
  name='ProcessScratchpadResp',
  full_name='wirepas.proto.gateway_api.ProcessScratchpadResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.ProcessScratchpadResp.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1189,
  serialized_end=1271,
)


_SETSCRATCHPADTARGETANDACTIONREQ = _descriptor.Descriptor(
  name='SetScratchpadTargetAndActionReq',
  full_name='wirepas.proto.gateway_api.SetScratchpadTargetAndActionReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.SetScratchpadTargetAndActionReq.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target_and_action', full_name='wirepas.proto.gateway_api.SetScratchpadTargetAndActionReq.target_and_action', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1274,
  serialized_end=1446,
)


_SETSCRATCHPADTARGETANDACTIONRESP = _descriptor.Descriptor(
  name='SetScratchpadTargetAndActionResp',
  full_name='wirepas.proto.gateway_api.SetScratchpadTargetAndActionResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.SetScratchpadTargetAndActionResp.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1448,
  serialized_end=1541,
)

_TARGETSCRATCHPADANDACTION.fields_by_name['action'].enum_type = _SCRATCHPADACTION
_TARGETSCRATCHPADANDACTION.fields_by_name['delay'].enum_type = _PROCESSINGDELAY
_TARGETSCRATCHPADANDACTION.oneofs_by_name['param'].fields.append(
  _TARGETSCRATCHPADANDACTION.fields_by_name['delay'])
_TARGETSCRATCHPADANDACTION.fields_by_name['delay'].containing_oneof = _TARGETSCRATCHPADANDACTION.oneofs_by_name['param']
_TARGETSCRATCHPADANDACTION.oneofs_by_name['param'].fields.append(
  _TARGETSCRATCHPADANDACTION.fields_by_name['raw'])
_TARGETSCRATCHPADANDACTION.fields_by_name['raw'].containing_oneof = _TARGETSCRATCHPADANDACTION.oneofs_by_name['param']
_GETSCRATCHPADSTATUSREQ.fields_by_name['header'].message_type = wp__global__pb2._REQUESTHEADER
_GETSCRATCHPADSTATUSRESP.fields_by_name['header'].message_type = wp__global__pb2._RESPONSEHEADER
_GETSCRATCHPADSTATUSRESP.fields_by_name['stored_scratchpad'].message_type = _SCRATCHPADINFO
_GETSCRATCHPADSTATUSRESP.fields_by_name['stored_status'].enum_type = _SCRATCHPADSTATUS
_GETSCRATCHPADSTATUSRESP.fields_by_name['stored_type'].enum_type = _SCRATCHPADTYPE
_GETSCRATCHPADSTATUSRESP.fields_by_name['processed_scratchpad'].message_type = _SCRATCHPADINFO
_GETSCRATCHPADSTATUSRESP.fields_by_name['target_and_action'].message_type = _TARGETSCRATCHPADANDACTION
_UPLOADSCRATCHPADREQ.fields_by_name['header'].message_type = wp__global__pb2._REQUESTHEADER
_UPLOADSCRATCHPADRESP.fields_by_name['header'].message_type = wp__global__pb2._RESPONSEHEADER
_PROCESSSCRATCHPADREQ.fields_by_name['header'].message_type = wp__global__pb2._REQUESTHEADER
_PROCESSSCRATCHPADRESP.fields_by_name['header'].message_type = wp__global__pb2._RESPONSEHEADER
_SETSCRATCHPADTARGETANDACTIONREQ.fields_by_name['header'].message_type = wp__global__pb2._REQUESTHEADER
_SETSCRATCHPADTARGETANDACTIONREQ.fields_by_name['target_and_action'].message_type = _TARGETSCRATCHPADANDACTION
_SETSCRATCHPADTARGETANDACTIONRESP.fields_by_name['header'].message_type = wp__global__pb2._RESPONSEHEADER
DESCRIPTOR.message_types_by_name['ScratchpadInfo'] = _SCRATCHPADINFO
DESCRIPTOR.message_types_by_name['TargetScratchpadAndAction'] = _TARGETSCRATCHPADANDACTION
DESCRIPTOR.message_types_by_name['GetScratchpadStatusReq'] = _GETSCRATCHPADSTATUSREQ
DESCRIPTOR.message_types_by_name['GetScratchpadStatusResp'] = _GETSCRATCHPADSTATUSRESP
DESCRIPTOR.message_types_by_name['UploadScratchpadReq'] = _UPLOADSCRATCHPADREQ
DESCRIPTOR.message_types_by_name['UploadScratchpadResp'] = _UPLOADSCRATCHPADRESP
DESCRIPTOR.message_types_by_name['ProcessScratchpadReq'] = _PROCESSSCRATCHPADREQ
DESCRIPTOR.message_types_by_name['ProcessScratchpadResp'] = _PROCESSSCRATCHPADRESP
DESCRIPTOR.message_types_by_name['SetScratchpadTargetAndActionReq'] = _SETSCRATCHPADTARGETANDACTIONREQ
DESCRIPTOR.message_types_by_name['SetScratchpadTargetAndActionResp'] = _SETSCRATCHPADTARGETANDACTIONRESP
DESCRIPTOR.enum_types_by_name['ScratchpadType'] = _SCRATCHPADTYPE
DESCRIPTOR.enum_types_by_name['ScratchpadStatus'] = _SCRATCHPADSTATUS
DESCRIPTOR.enum_types_by_name['ScratchpadAction'] = _SCRATCHPADACTION
DESCRIPTOR.enum_types_by_name['ProcessingDelay'] = _PROCESSINGDELAY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ScratchpadInfo = _reflection.GeneratedProtocolMessageType('ScratchpadInfo', (_message.Message,), {
  'DESCRIPTOR' : _SCRATCHPADINFO,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.ScratchpadInfo)
  })
_sym_db.RegisterMessage(ScratchpadInfo)

TargetScratchpadAndAction = _reflection.GeneratedProtocolMessageType('TargetScratchpadAndAction', (_message.Message,), {
  'DESCRIPTOR' : _TARGETSCRATCHPADANDACTION,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.TargetScratchpadAndAction)
  })
_sym_db.RegisterMessage(TargetScratchpadAndAction)

GetScratchpadStatusReq = _reflection.GeneratedProtocolMessageType('GetScratchpadStatusReq', (_message.Message,), {
  'DESCRIPTOR' : _GETSCRATCHPADSTATUSREQ,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.GetScratchpadStatusReq)
  })
_sym_db.RegisterMessage(GetScratchpadStatusReq)

GetScratchpadStatusResp = _reflection.GeneratedProtocolMessageType('GetScratchpadStatusResp', (_message.Message,), {
  'DESCRIPTOR' : _GETSCRATCHPADSTATUSRESP,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.GetScratchpadStatusResp)
  })
_sym_db.RegisterMessage(GetScratchpadStatusResp)

UploadScratchpadReq = _reflection.GeneratedProtocolMessageType('UploadScratchpadReq', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADSCRATCHPADREQ,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.UploadScratchpadReq)
  })
_sym_db.RegisterMessage(UploadScratchpadReq)

UploadScratchpadResp = _reflection.GeneratedProtocolMessageType('UploadScratchpadResp', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADSCRATCHPADRESP,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.UploadScratchpadResp)
  })
_sym_db.RegisterMessage(UploadScratchpadResp)

ProcessScratchpadReq = _reflection.GeneratedProtocolMessageType('ProcessScratchpadReq', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSSCRATCHPADREQ,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.ProcessScratchpadReq)
  })
_sym_db.RegisterMessage(ProcessScratchpadReq)

ProcessScratchpadResp = _reflection.GeneratedProtocolMessageType('ProcessScratchpadResp', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSSCRATCHPADRESP,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.ProcessScratchpadResp)
  })
_sym_db.RegisterMessage(ProcessScratchpadResp)

SetScratchpadTargetAndActionReq = _reflection.GeneratedProtocolMessageType('SetScratchpadTargetAndActionReq', (_message.Message,), {
  'DESCRIPTOR' : _SETSCRATCHPADTARGETANDACTIONREQ,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.SetScratchpadTargetAndActionReq)
  })
_sym_db.RegisterMessage(SetScratchpadTargetAndActionReq)

SetScratchpadTargetAndActionResp = _reflection.GeneratedProtocolMessageType('SetScratchpadTargetAndActionResp', (_message.Message,), {
  'DESCRIPTOR' : _SETSCRATCHPADTARGETANDACTIONRESP,
  '__module__' : 'otap_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.SetScratchpadTargetAndActionResp)
  })
_sym_db.RegisterMessage(SetScratchpadTargetAndActionResp)


# @@protoc_insertion_point(module_scope)



"""
    Set scratchpad target
    =====================

    .. Copyright:
        Copyright 2020 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

from .proto import GenericMessage

from .request import Request
from .response import Response

from .otap_helper import (
    set_scratchpad_target,
    parse_scratchpad_target,
    ScratchpadAction
)
from .gateway_result_code import GatewayResultCode
from .wirepas_exceptions import GatewayAPIParsingException


class SetScratchpadTargetAndActionRequest(Request):
    """
    SetScratchpadTargetAndActionRequest: request to set target scratchpad and action of a given sink

    Attributes:
        sink_id (str): id of the sink (dependant on gateway)
        target(dic): dictionary containing target scratchpad and associated action
            Dict keys are:
                action (ScratchpadAction): target action for network
                target_sequence (int): target sequence
                target_crc (int): target crc
                param(int): raw parameter for action
                delay(ProcessingDelay): delay for WITH_DELAY action
        req_id (int): unique request id
    """

    def __init__(self,
                 sink_id,
                 target,
                 req_id=None, **kwargs):
        super(SetScratchpadTargetAndActionRequest, self).__init__(sink_id, req_id, **kwargs)

        if target["action"].value > 255:
            raise ValueError("Wrong Target action")

        try:
            seq = target["target_sequence"]
            if seq is not None and seq > 255:
                raise ValueError("Wrong Target sequence")
        except KeyError:
            # Key is optional
            pass

        try:
            crc = target["target_crc"]
            if crc is not None and crc > 0xffff:
                raise ValueError("Wrong Target crc")
        except KeyError:
            # Key is optional
            pass

        try:
            param = target["param"]
            if param is not None and param > 255:
                raise ValueError("Wrong Action param")
        except KeyError:
            # Key is optional
            pass

        try:
            delay = target["delay"]
            if delay is not None and delay.value > 255:
                raise ValueError("Wrong Action param")
        except KeyError:
            # Key is optional
            pass

        self.target = target

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException(
                "Cannot parse SetScratchpadTargetAndActionRequest payload"
            )

        req = message.wirepas.set_scratchpad_target_and_action_req

        d = Request._parse_request_header(req.header)

        target = {}
        parse_scratchpad_target(req.target_and_action, target)

        return cls(sink_id=d["sink_id"],
                   target=target,
                   req_id=d["req_id"])

    @property
    def payload(self):
        message = GenericMessage()
        # Fill the request header
        req = message.wirepas.set_scratchpad_target_and_action_req
        self._load_request_header(req)

        set_scratchpad_target(req.target_and_action, self.target)

        return message.SerializeToString()


class SetScratchpadTargetAndActionResponse(Response):
    """
    SetScratchpadTargetAndActionResponse: Response to answer a SetScratchpadTargetAndActionRequest

      Attributes:
         req_id (int): unique request id that this Response is associated
         gw_id (str): gateway unique identifier
         res (GatewayResultCode): result of the operation
         sink_id (str): id of the sink (dependant on gateway)
     """

    def __init__(self, req_id, gw_id, res, sink_id, **kwargs):
        super(SetScratchpadTargetAndActionResponse, self).__init__(req_id, gw_id, res, sink_id, **kwargs)

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException("Cannot parse SetScratchpadTargetAndActionResponse payload")

        response = message.wirepas.set_scratchpad_target_and_action_resp

        d = Response._parse_response_header(response.header)

        return cls(d["req_id"], d["gw_id"], d["res"], d["sink_id"])

    @property
    def payload(self):
        message = GenericMessage()

        response = message.wirepas.set_scratchpad_target_and_action_resp
        self._load_response_header(response)

        return message.SerializeToString()



"""
    Config Helper
    =============

    .. Copyright:
        Copyright 2020 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

from .proto import ON, OFF, NodeRole


def convert_proto_role_to_wirepas(base, flags):
    """
    Maps the proto role enum to the correct Wirepas Mesh role

    Args:
        base (enum): protocol buffer enum
        flags (enum): protocol buffer enum

    Returns:
        Wirepas role as an int
    """
    base_proto_to_wirepas = dict(
        [
            (NodeRole.SINK, 1),
            (NodeRole.ROUTER, 2),
            (NodeRole.NON_ROUTER, 3),
        ]
    )

    flag_proto_to_wirepas = dict(
        [
            (NodeRole.LOW_LATENCY, 0x10),
            (NodeRole.AUTOROLE, 0x80),
        ]
    )

    wirepas_role = base_proto_to_wirepas[base]

    for flag in flags:
        wirepas_role += flag_proto_to_wirepas[flag]

    return wirepas_role


def convert_wirepas_role_to_proto(wirepas_role):
    """
    Maps a Wirepas role to a protobuff enum

    Args:
        wirepas_role (int): device role in WM network

    Returns:
        Tupple with protocol buffer base and flags
    """
    wirepas_base = wirepas_role & 0xF

    wirepas_base_to_proto = dict(
        [
            (1, NodeRole.SINK),
            (2, NodeRole.ROUTER),
            (3, NodeRole.NON_ROUTER),
        ]
    )

    flag_proto_to_wirepas = dict(
        [
            (0x10, NodeRole.LOW_LATENCY),
            (0x80, NodeRole.AUTOROLE),
        ]
    )

    base = wirepas_base_to_proto[wirepas_base]

    flags = []
    flags_val = wirepas_role & 0xF0
    for flag in flag_proto_to_wirepas:
        if flag & flags_val != 0:
            flags.append(flag_proto_to_wirepas[flag])

    return base, flags


def parse_optional_field(message_obj, field, dic, key):
    """
    Copies attribute into dic if it exists in the proto message

    Args:
        message_obj (proto): protocol buffer object
        field (str): the field to test for existence in message_obj
        dic (dict): the dictionary where to copy the value into
        key (str): the dictionary key to use

    """

    if message_obj.HasField(field):
        dic[key] = getattr(message_obj, field)


def set_optional_field(message_obj, field, dic, key):
    """
    Sets field in the protocol buffer object. If the object does not
    accept it or if the key is not in dic the error is ignored.

    Args:
        message_obj (proto): protocol buffer object
        field (str): the field to test for existence in message_obj
        dic (dict): the dictionary where to copy the value into
        key (str): the dictionary key to use

    """

    try:
        setattr(message_obj, field, dic[key])
    except KeyError:
        # Field is unknown, just skip it
        pass


def parse_config_rw(message_obj, dic):
    """
    Parses the message_obj for read and write (rw) fields and copies them into
    dic.

    Args:
        message_obj (proto): protocol buffer object
        dic (dict): the dictionary where to copy the rw fields

    """

    if message_obj.HasField("node_role"):
        dic["node_role"] = convert_proto_role_to_wirepas(
            message_obj.node_role.role, message_obj.node_role.flags
        )

    parse_optional_field(message_obj, "node_address", dic, "node_address")
    parse_optional_field(message_obj, "network_address", dic, "network_address")
    parse_optional_field(message_obj, "network_channel", dic, "network_channel")
    parse_optional_field(
        message_obj.app_config, "diag_interval_s", dic, "app_config_diag"
    )
    parse_optional_field(message_obj.app_config, "seq", dic, "app_config_seq")
    parse_optional_field(
        message_obj.app_config, "app_config_data", dic, "app_config_data"
    )
    parse_optional_field(message_obj, "channel_map", dic, "channel_map")

    if message_obj.HasField("sink_state"):
        dic["started"] = message_obj.sink_state == ON


def set_config_rw(message_obj, dic):
    """
    Sets the message_obj with the read and write (rw) fields present in dic.

    Args:
        message_obj (proto): protocol buffer object
        dic (dict): the dictionary where to copy the rw fields from

    """
    try:
        base, flags = convert_wirepas_role_to_proto(dic["node_role"])
        message_obj.node_role.role = base
        message_obj.node_role.flags.extend(flags)
    except KeyError:
        # Field is unknown, just skip it
        pass

    set_optional_field(message_obj, "node_address", dic, "node_address")
    set_optional_field(message_obj, "network_address", dic, "network_address")
    set_optional_field(message_obj, "network_channel", dic, "network_channel")

    set_optional_field(
        message_obj.app_config, "diag_interval_s", dic, "app_config_diag"
    )
    set_optional_field(message_obj.app_config, "seq", dic, "app_config_seq")

    try:
        message_obj.app_config.app_config_data = bytes(dic["app_config_data"])
    except KeyError:
        # Field is unknown, just skip it
        pass

    set_optional_field(message_obj, "channel_map", dic, "channel_map")

    try:
        if dic["started"]:
            message_obj.sink_state = ON
        else:
            message_obj.sink_state = OFF
    except KeyError:
        # Field is unknown, just skip it
        pass


def parse_config_keys(message_obj, dic):
    """
    Parses network keys.

    The keys are only available on a SinkNewConfig message.

    Args:
        message_obj (proto): protocol buffer object
        dic (dict): the dictionary where to copy the keys into
    """

    parse_optional_field(message_obj.keys, "cipher", dic, "cipher_key")
    parse_optional_field(message_obj.keys, "authentication", dic, "authentication_key")


def set_config_keys(message_obj, dic):
    """
    Sets network keys.

    The keys are only available on a SinkNewConfig message.

    Args:
        message_obj (proto): protocol buffer object
        dic (dict): the dictionary where to copy the keys from
    """
    set_optional_field(message_obj, "cipher", dic, "cipher_key")
    set_optional_field(message_obj, "authentication", dic, "authentication_key")


def parse_config_ro(message_obj, dic):
    """
    Sets the message_obj with the read only (ro) fields present in dic.

    Args:
        message_obj (proto): protocol buffer object
        dic (dict): the dictionary where to copy the ro fields from

    """
    parse_optional_field(
        message_obj.current_ac_range, "min_ms", dic, "current_ac_range_min"
    )
    parse_optional_field(
        message_obj.current_ac_range, "max_ms", dic, "current_ac_range_max"
    )

    parse_optional_field(message_obj.ac_limits, "min_ms", dic, "min_ac")
    parse_optional_field(message_obj.ac_limits, "max_ms", dic, "max_ac")

    parse_optional_field(message_obj, "max_mtu", dic, "max_mtu")

    parse_optional_field(message_obj.channel_limits, "min_channel", dic, "min_ch")
    parse_optional_field(message_obj.channel_limits, "max_channel", dic, "max_ch")
    parse_optional_field(message_obj, "hw_magic", dic, "hw_magic")
    parse_optional_field(message_obj, "stack_profile", dic, "stack_profile")
    parse_optional_field(message_obj, "app_config_max_size", dic, "app_config_max_size")
    parse_optional_field(message_obj, "are_keys_set", dic, "are_keys_set")

    if message_obj.HasField("firmware_version"):
        dic["firmware_version"] = [
            message_obj.firmware_version.major,
            message_obj.firmware_version.minor,
            message_obj.firmware_version.maint,
            message_obj.firmware_version.dev,
        ]


def set_config_ro(message_obj, dic):
    """
    Sets the message_obj with the read only (ro) fields present in dic.

    Args:
        message_obj (proto): protocol buffer object
        dic (dict): the dictionary where to copy the ro fields from

    """
    set_optional_field(message_obj.current_ac_range, "min_ms", dic, "min_ac_cur")
    set_optional_field(message_obj.current_ac_range, "max_ms", dic, "max_ac_cur")

    set_optional_field(message_obj.ac_limits, "min_ms", dic, "min_ac")
    set_optional_field(message_obj.ac_limits, "max_ms", dic, "max_ac")

    set_optional_field(message_obj, "max_mtu", dic, "max_mtu")

    set_optional_field(message_obj.channel_limits, "min_channel", dic, "min_ch")
    set_optional_field(message_obj.channel_limits, "max_channel", dic, "max_ch")

    set_optional_field(message_obj, "hw_magic", dic, "hw_magic")
    set_optional_field(message_obj, "stack_profile", dic, "stack_profile")
    set_optional_field(message_obj, "app_config_max_size", dic, "app_config_max_size")
    set_optional_field(message_obj, "are_keys_set", dic, "are_keys_set")

    try:
        message_obj.firmware_version.major = dic["firmware_version"][0]
        message_obj.firmware_version.minor = dic["firmware_version"][1]
        message_obj.firmware_version.maint = dic["firmware_version"][2]
        message_obj.firmware_version.dev = dic["firmware_version"][3]
    except KeyError:
        # Field is unknown, just skip it
        pass



"""
    Gateway result code
    ===================

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

# flake8: noqa

import enum
from .proto.error_pb2 import *


class GatewayResultCode(enum.Enum):
    """
    Class that represent all possible errors generated by a gateway
    Keep a one-to-one mapping with current protobuf errors to ease
    conversion
    """

    GW_RES_UNKNOWN_ERROR = UNKNOWN_ERROR_CODE
    GW_RES_OK = OK
    GW_RES_INTERNAL_ERROR = INTERNAL_ERROR
    GW_RES_INVALID_SINK_ID = INVALID_SINK_ID
    GW_RES_INVALID_ROLE = INVALID_ROLE
    GW_RES_INVALID_NETWORK_ADDRESS = INVALID_NETWORK_ADDRESS
    GW_RES_INVALID_NETWORK_CHANNEL = INVALID_NETWORK_CHANNEL
    GW_RES_INVALID_CHANNEL_MAP = INVALID_CHANNEL_MAP
    GW_RES_INVALID_NETWORK_KEYS = INVALID_NETWORK_KEYS
    GW_RES_INVALID_AC_RANGE = INVALID_AC_RANGE
    GW_RES_INVALID_SINK_STATE = INVALID_SINK_STATE
    GW_RES_INVALID_DEST_ADDRESS = INVALID_DEST_ADDRESS
    GW_RES_INVALID_DEST_ENDPOINT = INVALID_DEST_ENDPOINT
    GW_RES_INVALID_SRC_ENDPOINT = INVALID_SRC_ENDPOINT
    GW_RES_INVALID_QOS = INVALID_QOS
    GW_RES_INVALID_DATA_PAYLOAD = INVALID_DATA_PAYLOAD
    GW_RES_INVALID_SCRATCHPAD = INVALID_SCRATCHPAD
    GW_RES_INVALID_SCRATCHPAD_SIZE = INVALID_SCRATCHPAD_SIZE
    GW_RES_INVLAID_SEQUENCE_NUMBER = INVLAID_SEQUENCE_NUMBER
    GW_RES_INVALID_SEQUENCE_NUMBER = INVALID_SEQUENCE_NUMBER
    GW_RES_INVALID_REBOOT_DELAY = INVALID_REBOOT_DELAY
    GW_RES_INVALID_DIAG_INTERVAL = INVALID_DIAG_INTERVAL
    GW_RES_INVALID_APP_CONFIG = INVALID_APP_CONFIG
    GW_RES_INVALID_PARAM = INVALID_PARAM
    GW_RES_NO_SCRATCHPAD_PRESENT = NO_SCRATCHPAD_PRESENT
    GW_RES_ACCESS_DENIED = ACCESS_DENIED
    GW_RES_REQUEST_NEEDS_SINK_ID = REQUEST_NEEDS_SINK_ID
    GW_RES_INVALID_MAX_HOP_COUNT = INVALID_MAX_HOP_COUNT
    GW_RES_SINK_OUT_OF_MEMORY = SINK_OUT_OF_MEMORY
    GW_RES_SINK_TIMEOUT = SINK_TIMEOUT



"""
    .. Copyright:
       Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
       See file LICENSE for full license details.
"""

from .__about__ import (
    __author__,
    __author_email__,
    __classifiers__,
    __copyright__,
    __description__,
    __license__,
    __pkg_name__,
    __title__,
    __url__,
    __version__,
    __keywords__,
    __warning_msg__,
)

from .get_configs import GetConfigsRequest, GetConfigsResponse
from .get_gw_info import GetGatewayInfoRequest, GetGatewayInfoResponse
from .set_config import SetConfigRequest, SetConfigResponse
from .received_data import ReceivedDataEvent
from .status import StatusEvent, GatewayState
from .send_data import SendDataRequest, SendDataResponse
from .upload_scratchpad import UploadScratchpadRequest, UploadScratchpadResponse
from .process_scratchpad import ProcessScratchpadRequest, ProcessScratchpadResponse
from .get_scratchpad_status import GetScratchpadStatusRequest, GetScratchpadStatusResponse
from .set_scratchpad_target import SetScratchpadTargetAndActionRequest, SetScratchpadTargetAndActionResponse
from .gateway_result_code import GatewayResultCode
from .otap_helper import ScratchpadStatus, ScratchpadType, ScratchpadAction, ProcessingDelay
from .wirepas_exceptions import GatewayAPIParsingException

from google.protobuf.internal import api_implementation

# pylint: disable=locally-disabled, protected-access, wrong-import-order
try:
    if api_implementation._default_implementation_type == "python":
        print(__warning_msg__)
except AttributeError:
    print("Could not evaluate protobuf implementation type")


__all__ = [
    "__author__",
    "__author_email__",
    "__classifiers__",
    "__copyright__",
    "__description__",
    "__license__",
    "__pkg_name__",
    "__title__",
    "__url__",
    "__version__",
    "__keywords__",
    "__warning_msg__",
]



"""
    Get configs
    ===========

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

from .proto import GenericMessage

from .request import Request
from .response import Response

from .config_helper import (
    parse_config_rw,
    parse_config_ro,
    set_config_ro,
    set_config_rw,
)
from .wirepas_exceptions import GatewayAPIParsingException


class GetConfigsRequest(Request):
    """
    GetConfigsRequest: Request to obtain the configs of all sinks from a gateway

    Attributes:
        req_id (int): unique request id
    """

    def __init__(self, req_id=None, **kwargs):
        super(GetConfigsRequest, self).__init__(req_id=req_id, **kwargs)

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException("Cannot parse GetConfigsRequest payload")

        d = Request._parse_request_header(message.wirepas.get_configs_req.header)
        return cls(d["req_id"])

    @property
    def payload(self):
        message = GenericMessage()
        # Fill the request header
        get_config = message.wirepas.get_configs_req
        self._load_request_header(get_config)

        return message.SerializeToString()


class GetConfigsResponse(Response):
    """
    GetConfigsResponse: Response to answer a GetConfigsRequest

    Attributes:
        req_id (int): unique request id that this Response is associated
        gw_id (str): gw_id (str): gateway unique identifier
        res (GatewayResultCode): result of the operation
        configs (list): list of dictionnary containing a dict representing an attached sink
    """

    def __init__(self, req_id, gw_id, res, configs, **kwargs):
        super(GetConfigsResponse, self).__init__(req_id, gw_id, res, **kwargs)
        self.configs = configs

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException("Cannot parse GetConfigsResponse payload")

        response = message.wirepas.get_configs_resp

        d = Response._parse_response_header(response.header)

        configs = []

        for conf in response.configs:
            config = {}

            config["sink_id"] = conf.sink_id

            parse_config_rw(conf, config)
            parse_config_ro(conf, config)

            configs.append(config)

        return cls(d["req_id"], d["gw_id"], d["res"], configs)

    @property
    def payload(self):
        message = GenericMessage()

        response = message.wirepas.get_configs_resp
        self._load_response_header(response)

        for config in self.configs:
            conf = response.configs.add()
            conf.sink_id = config["sink_id"]

            set_config_rw(conf, config)
            set_config_ro(conf, config)

        return message.SerializeToString()



"""
    Set config
    ==========

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

from .proto import GenericMessage

from .request import Request
from .response import Response

from .config_helper import (
    parse_config_rw,
    parse_config_keys,
    set_config_rw,
    set_config_keys,
    parse_config_ro,
    set_config_ro,
)
from .wirepas_exceptions import GatewayAPIParsingException


class SetConfigRequest(Request):
    """
    SetConfigRequest: request to set config on a given sink

    Attributes:
        sink_id (str): id of the sink (dependant on gateway)
        new_config (dict): dictionnary containing the new config.
                Dict keys are:
                    node_role (int): Wirepas role
                    node_address (int)
                    network_address (int)
                    network_channel (int)
                    app_config_diag (int)
                    app_config_seq (int)
                    app_config_data (bytearray)
                    channel_map (int): discarded if not supported by sink
                    cipher_key (bytearray)
                    authentication_key (bytearray)
                    started (bool)

                Note: app_config_data/app_config_seq/app_config_data must all be defined to change one of them
                      only relevant keys for new config has to be defined
        req_id (int): unique request id
    """

    def __init__(self, sink_id, new_config, req_id=None, **kwargs):
        super(SetConfigRequest, self).__init__(req_id=req_id, **kwargs)
        self.sink_id = sink_id
        self.new_config = new_config

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException("Cannot parse SetConfigRequest payload")

        req = message.wirepas.set_config_req

        d = Request._parse_request_header(req.header)

        if (d["sink_id"] is not None) and (d["sink_id"] != req.config.sink_id):
            print("Warning: Mismatch in sink id for set_config_request")

        new_config = {}
        new_config["sink_id"] = req.config.sink_id
        parse_config_rw(req.config, new_config)
        parse_config_keys(req.config, new_config)

        return cls(req.config.sink_id, new_config, d["req_id"])

    @property
    def payload(self):
        message = GenericMessage()
        # Fill the request header
        set_config = message.wirepas.set_config_req
        self._load_request_header(set_config)

        set_config.config.sink_id = self.sink_id
        set_config_rw(set_config.config, self.new_config)
        set_config_keys(set_config.config.keys, self.new_config)

        return message.SerializeToString()


class SetConfigResponse(Response):
    """
    SetConfigResponse: Response to answer a SetConfigRequest

    Attributes:
        req_id (int): unique request id that this Response is associated
        gw_id (str): gateway unique identifier
        res (GatewayResultCode): result of the operation
        sink_id (str): id of the sink (dependant on gateway)
        config (dict): dictionnary containing the sink configuration after the setConfig request
    """

    def __init__(self, req_id, gw_id, res, sink_id, config, **kwargs):
        super(SetConfigResponse, self).__init__(req_id, gw_id, res, sink_id, **kwargs)
        # Config can be null in case of wrong sink_id for example
        self.config = config

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException("Cannot parse SetConfigResponse payload")

        response = message.wirepas.set_config_resp

        d = Response._parse_response_header(response.header)

        if d["sink_id"] != response.config.sink_id:
            print("Error: Mismatch in sink id for set_config_response")

        new_config = {}
        new_config["sink_id"] = response.config.sink_id
        parse_config_ro(response.config, new_config)
        parse_config_rw(response.config, new_config)

        return cls(d["req_id"], d["gw_id"], d["res"], d["sink_id"], new_config)

    @property
    def payload(self):
        message = GenericMessage()

        response = message.wirepas.set_config_resp
        self._load_response_header(response)

        response.config.sink_id = self.sink_id
        if self.config is not None:
            set_config_rw(response.config, self.config)
            set_config_ro(response.config, self.config)

        return message.SerializeToString()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_message.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import wp_global_pb2 as wp__global__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='data_message.proto',
  package='wirepas.proto.gateway_api',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12\x64\x61ta_message.proto\x12\x19wirepas.proto.gateway_api\x1a\x0fwp_global.proto\"\x82\x02\n\rSendPacketReq\x12\x38\n\x06header\x18\x01 \x02(\x0b\x32(.wirepas.proto.gateway_api.RequestHeader\x12\x1b\n\x13\x64\x65stination_address\x18\x02 \x02(\r\x12\x17\n\x0fsource_endpoint\x18\x03 \x02(\r\x12\x1c\n\x14\x64\x65stination_endpoint\x18\x04 \x02(\r\x12\x0b\n\x03qos\x18\x05 \x02(\r\x12\x0f\n\x07payload\x18\x06 \x02(\x0c\x12\x18\n\x10initial_delay_ms\x18\x07 \x01(\r\x12\x18\n\x10is_unack_csma_ca\x18\x08 \x01(\x08\x12\x11\n\thop_limit\x18\t \x01(\r\"K\n\x0eSendPacketResp\x12\x39\n\x06header\x18\x01 \x02(\x0b\x32).wirepas.proto.gateway_api.ResponseHeader\"\xcb\x02\n\x13PacketReceivedEvent\x12\x36\n\x06header\x18\x01 \x02(\x0b\x32&.wirepas.proto.gateway_api.EventHeader\x12\x16\n\x0esource_address\x18\x02 \x02(\r\x12\x1b\n\x13\x64\x65stination_address\x18\x03 \x02(\r\x12\x17\n\x0fsource_endpoint\x18\x04 \x02(\r\x12\x1c\n\x14\x64\x65stination_endpoint\x18\x05 \x02(\r\x12\x16\n\x0etravel_time_ms\x18\x06 \x02(\r\x12\x18\n\x10rx_time_ms_epoch\x18\x07 \x02(\x04\x12\x0b\n\x03qos\x18\x08 \x02(\r\x12\x0f\n\x07payload\x18\t \x01(\x0c\x12\x14\n\x0cpayload_size\x18\n \x01(\r\x12\x11\n\thop_count\x18\x0b \x01(\r\x12\x17\n\x0fnetwork_address\x18\x0c \x01(\x04'
  ,
  dependencies=[wp__global__pb2.DESCRIPTOR,])




_SENDPACKETREQ = _descriptor.Descriptor(
  name='SendPacketReq',
  full_name='wirepas.proto.gateway_api.SendPacketReq',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.SendPacketReq.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination_address', full_name='wirepas.proto.gateway_api.SendPacketReq.destination_address', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_endpoint', full_name='wirepas.proto.gateway_api.SendPacketReq.source_endpoint', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination_endpoint', full_name='wirepas.proto.gateway_api.SendPacketReq.destination_endpoint', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qos', full_name='wirepas.proto.gateway_api.SendPacketReq.qos', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='wirepas.proto.gateway_api.SendPacketReq.payload', index=5,
      number=6, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initial_delay_ms', full_name='wirepas.proto.gateway_api.SendPacketReq.initial_delay_ms', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_unack_csma_ca', full_name='wirepas.proto.gateway_api.SendPacketReq.is_unack_csma_ca', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hop_limit', full_name='wirepas.proto.gateway_api.SendPacketReq.hop_limit', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=325,
)


_SENDPACKETRESP = _descriptor.Descriptor(
  name='SendPacketResp',
  full_name='wirepas.proto.gateway_api.SendPacketResp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.SendPacketResp.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=402,
)


_PACKETRECEIVEDEVENT = _descriptor.Descriptor(
  name='PacketReceivedEvent',
  full_name='wirepas.proto.gateway_api.PacketReceivedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_address', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.source_address', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination_address', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.destination_address', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_endpoint', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.source_endpoint', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination_endpoint', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.destination_endpoint', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='travel_time_ms', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.travel_time_ms', index=5,
      number=6, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rx_time_ms_epoch', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.rx_time_ms_epoch', index=6,
      number=7, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qos', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.qos', index=7,
      number=8, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.payload', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload_size', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.payload_size', index=9,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hop_count', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.hop_count', index=10,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='network_address', full_name='wirepas.proto.gateway_api.PacketReceivedEvent.network_address', index=11,
      number=12, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=405,
  serialized_end=736,
)

_SENDPACKETREQ.fields_by_name['header'].message_type = wp__global__pb2._REQUESTHEADER
_SENDPACKETRESP.fields_by_name['header'].message_type = wp__global__pb2._RESPONSEHEADER
_PACKETRECEIVEDEVENT.fields_by_name['header'].message_type = wp__global__pb2._EVENTHEADER
DESCRIPTOR.message_types_by_name['SendPacketReq'] = _SENDPACKETREQ
DESCRIPTOR.message_types_by_name['SendPacketResp'] = _SENDPACKETRESP
DESCRIPTOR.message_types_by_name['PacketReceivedEvent'] = _PACKETRECEIVEDEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SendPacketReq = _reflection.GeneratedProtocolMessageType('SendPacketReq', (_message.Message,), {
  'DESCRIPTOR' : _SENDPACKETREQ,
  '__module__' : 'data_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.SendPacketReq)
  })
_sym_db.RegisterMessage(SendPacketReq)

SendPacketResp = _reflection.GeneratedProtocolMessageType('SendPacketResp', (_message.Message,), {
  'DESCRIPTOR' : _SENDPACKETRESP,
  '__module__' : 'data_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.SendPacketResp)
  })
_sym_db.RegisterMessage(SendPacketResp)

PacketReceivedEvent = _reflection.GeneratedProtocolMessageType('PacketReceivedEvent', (_message.Message,), {
  'DESCRIPTOR' : _PACKETRECEIVEDEVENT,
  '__module__' : 'data_message_pb2'
  # @@protoc_insertion_point(class_scope:wirepas.proto.gateway_api.PacketReceivedEvent)
  })
_sym_db.RegisterMessage(PacketReceivedEvent)


# @@protoc_insertion_point(module_scope)



"""
    Process scratchpad
    ===================

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

from .proto import GenericMessage

from .request import Request
from .response import Response
from .wirepas_exceptions import GatewayAPIParsingException


class ProcessScratchpadRequest(Request):
    """
    ProcessScratchpadRequest: request to process scratchpad on a given sink

    Attributes:
        sink_id (str): id of the sink (dependant on gateway)
        req_id (int): unique request id
    """

    def __init__(self, sink_id, req_id=None, **kwargs):
        super(ProcessScratchpadRequest, self).__init__(sink_id, req_id, **kwargs)

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException(
                "Cannot parse ProcessScratchpadRequest payload"
            )

        req = message.wirepas.process_scratchpad_req

        d = Request._parse_request_header(req.header)

        return cls(d["sink_id"], d["req_id"])

    @property
    def payload(self):
        message = GenericMessage()
        # Fill the request header
        req = message.wirepas.process_scratchpad_req
        self._load_request_header(req)

        return message.SerializeToString()


class ProcessScratchpadResponse(Response):
    """
    ProcessScratchpadResponse: Response to answer a ProcessScratchpadRequest

    Attributes:
        req_id (int): unique request id that this Response is associated
        gw_id (str): gw_id (str): gateway unique identifier
        res (GatewayResultCode): result of the operation
        sink_id (str): id of the sink (dependant on gateway)
    """

    def __init__(self, req_id, gw_id, res, sink_id, **kwargs):
        super(ProcessScratchpadResponse, self).__init__(
            req_id, gw_id, res, sink_id, **kwargs
        )

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException(
                "Cannot parse ProcessScratchpadResponse payload"
            )

        response = message.wirepas.process_scratchpad_resp

        d = Response._parse_response_header(response.header)

        return cls(d["req_id"], d["gw_id"], d["res"], d["sink_id"])

    @property
    def payload(self):
        message = GenericMessage()

        response = message.wirepas.process_scratchpad_resp
        self._load_response_header(response)

        return message.SerializeToString()



"""
    Upload scratchpad
    =================

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

from .proto import GenericMessage

from .request import Request
from .response import Response
from .wirepas_exceptions import GatewayAPIParsingException


class UploadScratchpadRequest(Request):
    """
    UploadScratchpadRequest: request to process scratchpad on a given sink

    Attributes:
        sink_id (str): id of the sink to upload scratchpad
        req_id (int): unique request id
        scratchpad (bytearray): scratchpad to upload (None to clear scratchpad)
    """

    def __init__(self, seq, sink_id, req_id=None, scratchpad=None, **kwargs):
        super(UploadScratchpadRequest, self).__init__(sink_id, req_id, **kwargs)
        self.seq = seq
        self.scratchpad = scratchpad

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException(
                "Cannot parse UploadScratchpadRequest payload"
            )

        req = message.wirepas.upload_scratchpad_req

        d = Request._parse_request_header(req.header)
        if req.HasField("scratchpad"):
            scratchpad = req.scratchpad
        else:
            # Clear the scratchpad
            scratchpad = None

        return cls(req.seq, d["sink_id"], d["req_id"], scratchpad)

    @property
    def payload(self):
        message = GenericMessage()
        # Fill the request header
        req = message.wirepas.upload_scratchpad_req
        self._load_request_header(req)

        req.seq = self.seq
        if self.scratchpad is not None:
            req.scratchpad = self.scratchpad

        return message.SerializeToString()


class UploadScratchpadResponse(Response):
    """
    UploadScratchpadResponse: Response to answer a UploadScratchpadRequest

    Attributes:
        req_id (int): unique request id that this Response is associated
        gw_id (str): gw_id (str): gateway unique identifier
        res (GatewayResultCode): result of the operation
        sink_id (str): id of the sink (dependant on gateway)
    """

    def __init__(self, req_id, gw_id, res, sink_id, **kwargs):
        super(UploadScratchpadResponse, self).__init__(
            req_id, gw_id, res, sink_id, **kwargs
        )

    @classmethod
    def from_payload(cls, payload):
        message = GenericMessage()
        try:
            message.ParseFromString(payload)
        except Exception:
            # Any Exception is promoted to Generic API exception
            raise GatewayAPIParsingException(
                "Cannot parse UploadScratchpadResponse payload"
            )

        response = message.wirepas.upload_scratchpad_resp

        d = Response._parse_response_header(response.header)

        return cls(d["req_id"], d["gw_id"], d["res"], d["sink_id"])

    @property
    def payload(self):
        message = GenericMessage()

        response = message.wirepas.upload_scratchpad_resp
        self._load_response_header(response)

        return message.SerializeToString()



"""
    Wirepas exceptions
    ==================

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""


class GatewayAPIParsingException(Exception):
    """
    Wirepas Gateway API generic Exception
    """



"""
    Gateway result code
    ===================

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""

# flake8: noqa
from .proto.otap_message_pb2 import BLANK, PRESENT, PROCESS, SUCCESS, NEW, ERROR
from .proto.otap_message_pb2 import NO_OTAP, PROPAGATE_ONLY, PROPAGATE_AND_PROCESS, PROPAGATE_AND_PROCESS_WITH_DELAY, LEGACY_OTAP
from .proto.otap_message_pb2 import UNKNOWN_DELAY, TEN_MINUTES, THIRTY_MINUTES, ONE_HOUR, SIX_HOURS, ONE_DAY, TWO_DAYS, FIVE_DAYS

import enum


def parse_optional_field(message_obj, field, dic, key):
    """
    Copies attribute into dic if it exists in the proto message

    Args:
        message_obj (proto): protocol buffer object
        field (str): the field to test for existence in message_obj
        dic (dict): the dictionary where to copy the value into
        key (str): the dictionary key to use

    """

    if message_obj.HasField(field):
        dic[key] = getattr(message_obj, field)


def set_optional_field(message_obj, field, dic, key):
    """
    Sets field in the protocol buffer object. If the object does not
    accept it or if the key is not in dic the error is ignored.

    Args:
        message_obj (proto): protocol buffer object
        field (str): the field to test for existence in message_obj
        dic (dict): the dictionary where to copy the value into
        key (str): the dictionary key to use

    """

    try:
        setattr(message_obj, field, dic[key])
    except KeyError:
        # Field is unknown, just skip it
        pass


class ScratchpadType(enum.Enum):
    SCRATCHPAD_TYPE_BLANK = BLANK
    SCRATCHPAD_TYPE_PRESENT = PRESENT
    SCRATCHPAD_TYPE_PROCESS = PROCESS


class ScratchpadStatus(enum.Enum):
    SCRATCHPAD_STATUS_SUCCESS = SUCCESS
    SCRATCHPAD_STATUS_NEW = NEW
    SCRATCHPAD_STATUS_ERROR = ERROR


class ScratchpadAction(enum.Enum):
    ACTION_NO_OTAP = NO_OTAP
    ACTION_PROPAGATE_ONLY = PROPAGATE_ONLY
    ACTION_PROPAGATE_AND_PROCESS = PROPAGATE_AND_PROCESS
    ACTION_PROPAGATE_AND_PROCESS_WITH_DELAY = PROPAGATE_AND_PROCESS_WITH_DELAY
    ACTION_LEGACY_OTAP = LEGACY_OTAP


class ProcessingDelay(enum.Enum):
    DELAY_UNKNOWN = UNKNOWN_DELAY
    DELAY_TEN_MINUTES =  TEN_MINUTES
    DELAY_THIRTY_MINUTES = THIRTY_MINUTES
    DELAY_ONE_HOUR = ONE_HOUR
    DELAY_SIX_HOURS = SIX_HOURS
    DELAY_ONE_DAY = ONE_DAY
    DELAY_TWO_DAYS = TWO_DAYS
    DELAY_FIVE_DAYS = FIVE_DAYS


def parse_scratchpad_info(message_obj, dic):
    dic["len"] = message_obj.len
    dic["crc"] = message_obj.crc
    dic["seq"] = message_obj.seq


def set_scratchpad_info(message_obj, dic):
    message_obj.len = dic["len"]
    message_obj.crc = dic["crc"]
    message_obj.seq = dic["seq"]


def parse_scratchpad_target(message_obj, dic):
    dic["action"] = ScratchpadAction(message_obj.action)
    parse_optional_field(message_obj, "target_sequence", dic, "target_sequence")
    parse_optional_field(message_obj, "target_crc", dic, "target_crc")
    # Only one of the following will be set
    param = message_obj.WhichOneof("param")
    if param == "delay":
        dic["delay"] = ProcessingDelay(message_obj.delay)
    elif param == "raw":
        dic["param"] = message_obj.raw


def set_scratchpad_target(message_obj, dic):
    message_obj.action = dic["action"].value
    set_optional_field(message_obj, "target_sequence", dic, "target_sequence")
    set_optional_field(message_obj, "target_crc", dic, "target_crc")
    # Only one of the following key should be set
    try:
        message_obj.delay = dic["delay"].value
    except KeyError:
        # try the raw version
        set_optional_field(message_obj, "raw", dic, "param")



"""
    Event
    =====

    .. Copyright:
        Copyright 2019 Wirepas Ltd under Apache License, Version 2.0.
        See file LICENSE for full license details.
"""
import random
from .proto import EventHeader


class Event(object):
    """
    Event

    Attributes:
        gw_id (str): gateway unique identifier
        sink_id(str): sink identifier
        event_id(int): event unique id (random value generated if None)
    """

    # pylint: disable=unused-argument
    def __init__(self, gw_id, sink_id=None, event_id=None, **kwargs):

        super(Event, self).__init__()
        self.gw_id = gw_id
        self.sink_id = sink_id
        if event_id is None:
            event_id = random.getrandbits(64)
        self.event_id = event_id

    def __str__(self):
        return str(self.__dict__)

    @property
    def payload(self):
        """ Implement how to serialize child Event classes """
        raise NotImplementedError()

    def _load_event_header(self, event):
        header = event.header
        header.gw_id = str(self.gw_id)
        header.event_id = self.event_id

        if self.sink_id is not None:
            header.sink_id = str(self.sink_id)

    @staticmethod
    def _parse_event_header(header):
        """
        Parses the header details from a protobuff message

        Args:
            header (proto): proto buff message

        Returns:
            A dictionary with the header details
        """

        d = dict()
        d["gw_id"] = header.gw_id
        d["event_id"] = header.event_id
        if header.HasField("sink_id"):
            d["sink_id"] = header.sink_id
        else:
            d["sink_id"] = None

        return d
